#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pylint: skip-file

# CAVEAT UTILITOR
#
# This file was automatically generated by Grako.
#
#    https://pypi.python.org/pypi/grako/
#
# Any changes you make to it will be overwritten the next time
# the file is generated.

from __future__ import (
    print_function,
    division,
    absolute_import,
    unicode_literals
)
from grako.parsing import graken, Parser
from grako.util import re, RE_FLAGS, generic_main  # noqa
from .lexer import Lambda, Var, Apply, Alias

__version__ = (2016, 6, 7, 6, 14, 58, 1)

__all__ = [
    'LambdaParser',
    'LambdaSemantics'
]

KEYWORDS = set([])
definitions = {}


class LambdaParser(Parser):
    def __init__(self,
                 whitespace=None,
                 nameguard=None,
                 comments_re=None,
                 eol_comments_re=None,
                 ignorecase=None,
                 left_recursion=True,
                 keywords=KEYWORDS,
                 namechars='',
                 **kwargs):
        super(LambdaParser, self).__init__(
            whitespace=whitespace,
            nameguard=nameguard,
            comments_re=comments_re,
            eol_comments_re=eol_comments_re,
            ignorecase=ignorecase,
            left_recursion=left_recursion,
            keywords=keywords,
            namechars=namechars,
            **kwargs
        )

    @graken()
    def _program_(self):

        def block0():
            self._line_()
        self._closure(block0)

    @graken()
    def _line_(self):
        with self._group():
            with self._choice():
                with self._option():
                    self._define_()
                with self._option():
                    self._function_()
                self._error('no available options')

    @graken()
    def _define_(self):
        self._token('@')
        self._alias_()
        self._token(':')
        self._function_()

    @graken()
    def _function_(self):
        with self._choice():
            with self._option():
                self._lambda_()
            with self._option():
                self._application_()
            self._error('no available options')

    @graken()
    def _lambda_(self):
        with self._group():
            with self._choice():
                with self._option():
                    self._token('\u03bb')
                with self._option():
                    self._token('\\')
                self._error('expecting one of: \\ \u03bb')
        self._variable_()
        self._token('.')
        self._expr_()

    @graken()
    def _application_(self):
        self._token('(')
        self._expr_()
        self._expr_()
        self._token(')')

    @graken()
    def _expr_(self):
        with self._group():
            with self._choice():
                with self._option():
                    self._lambda_()
                with self._option():
                    self._application_()
                with self._option():
                    self._variable_()
                with self._option():
                    self._alias_()
                self._error('no available options')

    @graken()
    def _variable_(self):
        self._pattern(r'[a-z]')

    @graken()
    def _alias_(self):
        self._pattern(r'[_A-Z][_A-Z0-9]*')


class LambdaSemantics(object):
    def program(self, ast):
        return ast, definitions

    def line(self, ast):
        return ast

    def define(self, ast):
        definitions[ast[1].name] = ast[3]
        return None

    def function(self, ast):
        return ast

    def lambda_(self, ast):
        return Lambda(ast[1], ast[3])

    def application(self, ast):
        return Apply(ast[1], ast[2])

    def expr(self, ast):
        return ast

    def variable(self, ast):
        return Var(ast.encode('utf-8'))

    def alias(self, ast):
        return Alias(ast)



#!venv/bin/python
"""
This includes the Lexical Element classes for:
Lambda -- a lambda function
Var -- a single variable
Apply -- an Expression application of two Lambda's or Apply's
Alias -- a meta-variable containing a Lambda from it's alias
Helper methods are included as well.
"""
from __future__ import print_function


class Lambda(object):
    """
    A Lambda Function in lambda calculus
    """
    def __init__(self, arg, body):
        """
        Lambda C'tor
        :param arg: Variable that is supplied as the function name.
        :param body: Lexical Element to be given as the function's body
        """
        if arg is None or body is None:
            self.body = Var(u'x')
            self.arg = Var(u'x')
        else:
            self.body = body
            self.arg = arg

    def evalstr(self):
        """
        Creates a string that can be used in python's eval to create a lambda
        :return: The string to make lambda from eval
        """
        return '(lambda ' + self.arg.evalstr() + ': ' \
               + self.body.evalstr() + ')'


class Apply(object):
    """
    A Expression application in lambda calculus
    """
    def __init__(self, func, arg):
        """
        Apply C'tor
        :param func: The function forming the expression
        :param arg: The argument to be applied to the function
        """
        self.func = func
        self.arg = arg

    def evalstr(self):
        """
        Creates a string that can be used in python's eval to create a function
        :return: The string to make function from eval
        """
        return self.func.evalstr() + '(' + self.arg.evalstr() + ')'


class Var(object):
    """
    A Variable in lambda calculus
    """
    def __init__(self, name):
        """
        Var C'tor
        :param name: the name of the variable.
        """
        self.name = name

    def evalstr(self):
        """
        Creates a string that can be used in python's eval to create a variable
        :return: The string to make variable from eval
        """
        return self.name


class Alias(object):
    """
    Meta-Variable containing a lambda caluclus function
    Used for function definition substitution in the repl
    """
    def __init__(self, name):
        """
        Alias C'tor
        :param name: The named alias
        """
        self.name = name


__types = {
    "V": Var,
    "L": Lambda,
    "A": Apply,
}


def recursive_none_filter(none_list):
    """
    filters None objects from a list and any inner-lists recursively
    :param none_list: the list to filter
    :return: if none_list is a list,
     then the filtered none_list
     else none_list itself
    """
    if isinstance(none_list, list):
        values = [recursive_none_filter(i)
                  for i in none_list if recursive_none_filter(i) is not None]
        if len(values) is not 0:
            return values
        return None
    return none_list


def replace_alias(obj, defs):
    """
    Recursively replaces aliases with their appropriate lambda functions
    :param obj: The main Lexical element to replace from
    :param defs: The dict of valid aliases and their values
    :return: the mapped lexical element.
    """
    if isinstance(obj, Lambda):
        body, rep = replace_alias(obj.body, defs)
        return Lambda(obj.arg, body), rep
    elif isinstance(obj, Apply):
        func, rep1 = replace_alias(obj.func, defs)
        arg, rep2 = replace_alias(obj.arg, defs)
        return Apply(func, arg), rep1 or rep2
    elif isinstance(obj, Alias):
        if obj.name in defs:
            return defs[obj.name], True
    return obj, False



"""
The Main Repl for ChurchREPL
"""
import sys
from churchrepl import set_args, repl_eval


def main(args):
    """
    The main repl runner
    :param args: args supplied to set_args
    :return: None
    """
    repl_eval("> ", set_args(args))

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python
from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    url='https://bitbucket.org/Autological/churchrepl',
    name='churchrepl',
    version='0.2.0',
    description=('A simple REPL for lambda calculus'),
    long_description=long_description,
    author='Luke Smith',
    author_email='lsmith@zenoscave.com',
    license='BSD',
    entry_points={'console_scripts': [
        'churchrepl = churchrepl.__main__:main',
    ]},
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows :: Windows 7',
    ],
    packages=['churchrepl', 'churchrepl.repl'],
    install_requires=['grako', 'future'],
)


"""
ChurchRepl is a lambda calculus REPL built for study of church's theses
"""

from __future__ import print_function
import argparse
from builtins import input, str

from churchrepl.repl import parse, lex


def _place(val, verbose, args):
    """
    Print wrapper to handle verbose statements
    :param val: value to print
    :param verbose: optional verbose value to print
    :return: None
    """
    if args.verbose:
        val += " --> " + verbose
    print(val)


# def _add_(body, func):
#     """
#     Underlying Church enumeration adder
#     :param body: the lambda body
#     :param func: the lambda itself
#     :return: given the value of the application on the body,
#     returns the value if it is a lambda itself, else the value + 1
#     """
#     val = func(body) if body is not None else -1
#     if hasattr(val, '__call__'):
#         return val
#     else:
#         return val + 1
#
#
# def _add(body, func):
#     """
#     Wrapper Church enumeration adder
#     :param body: the lambda body
#     :param func: the lambda itself
#     :return: __add(x, a) or 0
#     """
#     return _add_(body, func) if body is not None else 0
#
#
# def _to_int(lambda_expr):
#     """
#     Church enumeration converter
#     :param lambda_expr: the lambda to convert
#     :return: x's enumerated value.
#     """
#     return lambda_expr + 1


def set_args(args):
    """
    Creates the argset from an argumentParser
    :param args: the list of arg strings to parse
    :return: an argument object containing the arg vals
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", action="store_true")
    parser.add_argument("-f", "--file", nargs="+", type=str)
    return parser.parse_args(args)


def repl_eval(prompt, args):
    """
    Runs a Read Eval Print Loop.
    ^D exits gracefully from the loop.
    files may be loaded into the repl
    verbosity may be included
    :param prompt: the command line prompt to print
    :param args: an args object, typically from set_args(args)
    :return: N/A
    """

    if args.file is not None and len(args.file) is not 0:
        program = '\n'.join([open(filename).read() for filename in args.file])
        ast, defs = parse(program)
        if ast is not None:
            try:
                ast, defs = parse(program)
                lexes = lex(ast, defs)
                i = 0
                for i in defs:
                    _place(i, defs[i].evalstr(), args)
                for j in [eval(lexical.evalstr)(lambda x: x+1)(0)
                          for lexical in lexes]:
                    _place(str(j), str(lexes[i].evalstr()), args)
                    i += 1
            except Exception as err:
                _place('!!!', str(err), args)

    try:
        while True:
            program = input(prompt)
            try:
                ast, defs = parse(program)
                lexes = lex(ast, defs)
                i = 0
                for j in [eval(lexical.evalstr)(lambda x: x+1)(0)
                          for lexical in lexes]:
                    _place(str(j), str(lexes[i].evalstr()), args)
                    i += 1
            except Exception as err:
                _place('!!!', str(err), args)
    except EOFError:
        pass



#!venv/bin/python
"""
The underlying semantic, syntax and lexical analysis module
"""

from __future__ import print_function
from grako.buffering import Buffer
from .parser import LambdaParser, LambdaSemantics
from .lexer import replace_alias

parser = LambdaParser()


def parse(program):
    """
    Parse a string into a list of lexical objects
    :param program: string to parse
    :return: Lexcial object lists
    """
    return parser.parse(Buffer(program, nameguard=False), 'program',
                        semantics=LambdaSemantics())


def lex(ast, definitions):
    """
    Lexical object emplacement of aliases
    :param ast: the lexical objects to replace aliases
    :param definitions: valid aliases
    :return: The formatted lexical object list
    """
    ret = []
    for line in ast:
        line, rep = replace_alias(line, definitions)
        while rep:
            line, rep = replace_alias(line, definitions)
        ret.append(line)
    return ret
